@@ -3,7 +3,10 @@ use strict;
 use warnings;
 use IO::Socket::SIPC;
 
-my $sipc = IO::Socket::SIPC->new( favorite => 'IO::Socket::INET' );
+my $sipc = IO::Socket::SIPC->new(
+   favorite      => 'IO::Socket::INET',
+   use_check_sum => 1,
+);
 
 $sipc->connect(
    LocalAddr  => 'localhost',
